#include "DEMTTT_Posters_ttt.h"

/*------------------------------------------------------------------------
 *  poster reading / writing tools
 */

/**
 *  \brief Convert a DTM_P3D poster into a DEM
 *  \param poster : the input poster
 *  \return a new DEM corresponding to the input poster
 *
 *  A new instance of DEM object is created
 */
DEM* DTM_P3D_POSTER_to_DEM(const DTM_P3D_POSTER& poster)
{
  DEM* dem = new DEM(poster.nbCols,
		     poster.nbLines,
		     poster.xOrigin,
		     poster.yOrigin,
		     poster.zOrigin,
		     poster.xScale,
		     poster.yScale,
		     poster.zScale);

  int NB_COLS = dem->nbCols();
  int NB_LINES = dem->nbLines();

  for(int i=0; i<NB_COLS; i++)
    {
      for(int j=0; j<NB_LINES; j++)
	{
// 	  if(poster.nb_points[i][j] != 0)
// 	    {
// 	      dem->setCell(i, 
// 			   j, 
// 			   poster.zfloat[i][j], 
// 			   poster.nb_points[i][j], 
// 			   poster.sigma_z[i][j], 
// 			   poster.confidence[i][j]);
// 	    }
	  if(poster.state[i][j] == DTM_CELL_FILLED)
	    {
	      dem->setCell(i, 
			   j, 
			   poster.zfloat[i][j], 
			   1, 
			   0, 
			   0);
	    }
	  else // if(poster.state[i][j] != DTM_CELL_EMPTY)
	    {
	      dem->setEmptyCell(i, j);
	    }
	}
    }

  return dem;
}

/*
 *  \brief update the local DEM of the Mana robot
 *  \return the resulting DEM if the process succeed, NULL else
 */
DEM* updateDEM(const POSTER_ID& demPosterId)
{
  // read the current DEM
  DTM_P3D_POSTER dem;
  if(MPC_DEMTreatmentDTM_P3D_POSTERPosterRead (demPosterId, &dem) == ERROR)
    {
      return NULL;
    }

  // convert it into a mpc::DEM class
  DEM* demRes = DTM_P3D_POSTER_to_DEM(dem);
  return demRes;
}

/*
 *  \brief Fill the distance to obstacles map poster from a mpc distmap
 *  \param distMap : the input mpc distmap
 */
int fillDistMapPoster(DistObsMap* distMap)
{
  if(distMap == NULL) return -1;

  SDI_F->distObsMap.dimX = distMap->nbCols();
  SDI_F->distObsMap.dimY = distMap->nbLines();

  SDI_F->distObsMap.xOrigin = distMap->xOrigin();
  SDI_F->distObsMap.yOrigin = distMap->yOrigin();
  SDI_F->distObsMap.xScale = distMap->xScale();
  SDI_F->distObsMap.yScale = distMap->yScale();

  for(int i=0; i < SDI_F->distObsMap.dimX; i++)
    {
      for(int j=0; j < SDI_F->distObsMap.dimY; j++)
	{
	  obs2D_cell_t cellSt;
	  int distX;
	  int distY;
	  int nearX;
	  int nearY;
	  distMap->getCellInfo(i, j, cellSt, distX, distY, nearX, nearY);

 	  SDI_F->distObsMap.cellState[i][j] = cellSt;

	  SDI_F->distObsMap.dist[i][j].x = distX;
	  SDI_F->distObsMap.dist[i][j].y = distY;

// 	  SDI_F->distObsMap.near[i][j].x = nearX;
// 	  SDI_F->distObsMap.near[i][j].y = nearY;
	}
    }

  SDI_F->distObsMap.thresSlope = distMap->thresSlope();
  SDI_F->distObsMap.thresVarSlope = distMap->thresVarSlope();
  SDI_F->distObsMap.safeDist = distMap->safeDist();

 //   distMap->saveIntoPPM("DistMap_in_poster.ppm", -15, 22);
  return 0;
}
